/* 
    qasn1 - Asn1BerModel - Model/View Model class for qasn1 library
    Copyright(C) 2010 - Andrius da Costa Ribas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef ASN1BERMODEL_H
#define ASN1BERMODEL_H

#include "Asn1BerNode.h"

#include <QAbstractItemModel>
#include <QModelIndex>
#include <QVariant>

class Asn1BerModelPrivate;
class QASN1_EXPORT Asn1BerModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    enum Roles { TagRole = Qt::UserRole + 1, DefinedLengthRole, LengthRole, ValueRole, ClassRole, ConstructedRole };
    Asn1BerModel(Asn1BerNode* node, QObject *parent = 0);
    virtual ~Asn1BerModel();
    
    // read functions
    Qt::ItemFlags flags ( const QModelIndex & index ) const;
    QVariant data ( const QModelIndex & index, int role = Qt::DisplayRole ) const;
    QVariant headerData ( int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;
    int rowCount ( const QModelIndex & parent = QModelIndex() ) const;
    int columnCount ( const QModelIndex & parent = QModelIndex() ) const;
    
    // edit functions
    bool setData ( const QModelIndex & index, const QVariant & value, int role = Qt::EditRole );
    bool setHeaderData ( int section, Qt::Orientation orientation, const QVariant & value, int role = Qt::EditRole );
    
    // insertion functions
    bool insertRows ( int row, int count, const QModelIndex & parent = QModelIndex() );
    bool removeRows ( int row, int count, const QModelIndex & parent = QModelIndex() );
    
    
    QModelIndex index ( int row, int column, const QModelIndex & parent = QModelIndex() ) const;
    QModelIndex parent ( const QModelIndex & index ) const;
    
private:
    Asn1BerModelPrivate* d;
};

#endif
